"""
fpi10h
* THIS VERSION USES GRIDDED DEAD FUEL MOISTURE DATA FROM Matt Jolly
* NOTE THAT WATER MUST HAVE A VALUE OF 22 IN THE NFDRFUEL MAP
* Uses relative greenness and 10 hour timelag fuel moisture to estimate fire
    potential.
* It uses extinction moisture to normalize 10 hr moisture
* It uses Andres method to calc deadfrac to avoid a discontinuity
* It uses a maximum live ratio map rather than getting these values from
  a fuel model map.

Selected variable definitions:
    extm:     moisture content of dead fuel at which fire will no longer
                spread
    extmois:  dead fuel extinction moisture for the fuel model
    fm:       NFDRS fuel model (This is used just to get barren, ag, etc)
    fpi:      fire potential index
    lr:       live ratio as a fraction (proportion of max live
                that is currently live)
    dr:   proportion of fuel that is dead
    maxlr:    maximum ratio of live fuel load to total fuel load
    outval:   integer fpi value to be output
    rgfrac:   relative greenness as a fraction rather than a percentage
    rgval:    relative greenness value
    deadfrac:  ten hour moisture as a fraction of the extinction moisture
    tenhrval: 10 hour dead fuel moisture value
    rainmap: precipitation in mm
    ws: windspeed in knots (converted to mph in the code)
    dbmap: dry bulb (air) temperature in degrees F

Programmer: Bob Burgan
Date last revised: 1/16/2013
Updated on:
    12/17/2018 - OIMOEN - Code added to accomodate changes made by B. Burgan
    12/19/2018 - OIMOEN - bug fix - added code to use snow map
    02/24/2020 - CYOUNG - Code changes based on BBurgan's requested for snow
                            and clouds removal and BBurgan's source code.
    04/21/2020 - CYOUNG - additional code updated requested by BBurgan to
                            make WFPI comparable with FPI.
    05/13/2020 - CYOUNG - Moved ROWS and COLS to constants module.
"""
import argparse
import numpy

# import PPG modules
from lib import constants


def fpi10h_mso(param_file, verbose=False):

    # Load input file names
    with open(param_file, 'r') as fh:
        file_names = [x.strip() for x in fh.readlines()]

    # Open input data
    # Relative greenness map
    greenness = numpy.fromfile(file_names[0], dtype=numpy.uint8)
    # Maximum live ratio map
    liveratio = numpy.fromfile(file_names[1], dtype=numpy.uint8)
    # Extinction moisture image
    extm = numpy.fromfile(file_names[2], dtype=numpy.uint8)
    # Fuel model map
    fuel_model = numpy.fromfile(file_names[3], dtype=numpy.uint8)
    # Wind speed reduction
    windreduction = numpy.fromfile(file_names[4], dtype=numpy.uint8)
    # Ten hour moisture map
    ten_hr = numpy.fromfile(file_names[5], dtype=numpy.uint8)
    # Rain map (in mm)
    rainmap = numpy.fromfile(file_names[6], dtype=numpy.uint8)
    # Wind speed
    ws = numpy.fromfile(file_names[7], dtype=numpy.uint8)
    # Dry bulb temperature in degrees F
    dbmap = numpy.fromfile(file_names[8], dtype=numpy.int8)

    # Output data
    out_filename = file_names[9]
    fpi_file = numpy.zeros((constants.ROWS * constants.COLS,),
                           dtype=numpy.uint8)

    fm_gt_0 = fuel_model > 0
    fm_lt_22 = fuel_model < 22
    fm_ne_13 = fuel_model != 13
    valid_pixel_mask = fm_gt_0 & fm_lt_22 & fm_ne_13

    # Prevent calc value being too low
    for i in range(0, constants.ROWS * constants.COLS):
        ten_hr[i] += (extm[i] - ten_hr[i]) * (rainmap[i] / 13.0)
        if ten_hr[i] > extm[i]:
            ten_hr[i] = extm[i]
        if dbmap[i] < 45:
            ten_hr[i] += ((45.0 - dbmap[i]) / 15.0) * (extm[i] - ten_hr[i])
        if ten_hr[i] > extm[i]:
            ten_hr[i] = extm[i]

    ten_hr[ten_hr < 2] = 2

    # Greenness as a fraction
    double_greenness = greenness / 100.0
    double_greenness[double_greenness > 1.0] = 1.0
    double_greenness[double_greenness < 0.0] = 0.0

    # live load ratio
    lr = liveratio / 100.0 * double_greenness

    # dead load ratio
    dr = 1.0 - lr

    # frac10
    deadfrac = numpy.ones((constants.ROWS * constants.COLS,),
                          dtype=numpy.float64)

    # Minimum 10 hour moisture
    x1 = 2.0
    # Distance between outer tangent points and
    # (MinMoist, 0) and (Extmoint, 1)
    dist = 0.5

    alpha = numpy.arctan(1.0 / (extm - x1))
    # Radious of a circle
    r = dist / (numpy.sin(alpha / 2.0) / numpy.cos(alpha / 2.0))

    # Tangent point to the left of the minimum moisture
    x1a = x1 - dist
    # Tangent point to the left of the extinction moisture
    x1b = x1 + dist * numpy.cos(alpha)
    # Tangent point to the right of the minimum moisture
    x2a = extm - dist * numpy.cos(alpha)
    # Tangent point to the right of the extinction moisture
    x2b = extm + dist

    x1b_val = r - numpy.sqrt(r * r - numpy.power(ten_hr - x1a, 2))
    x2a_val = (ten_hr - x1) / (extm - x1)
    x2b_val = 1.0 - r + numpy.sqrt(r * r - numpy.power(ten_hr - x2b, 2))

    x1a_mask = ten_hr < x1a
    x1b_mask = numpy.logical_and(ten_hr < x1b, numpy.logical_not(x1a_mask))
    x2a_mask = numpy.logical_and(ten_hr < x2a, numpy.logical_and(
        numpy.logical_not(x1a_mask), numpy.logical_not(x1b_mask)))
    x2b_mask = numpy.logical_and(
        ten_hr < x2b,
        numpy.logical_and(
            numpy.logical_not(x1a_mask),
            numpy.logical_and(
                numpy.logical_not(
                    x1b_mask),
                numpy.logical_not(x2a_mask))))

    deadfrac[x1a_mask] = 0.0
    deadfrac[x1b_mask] = x1b_val[x1b_mask]
    deadfrac[x2a_mask] = x2a_val[x2a_mask]
    deadfrac[x2b_mask] = x2b_val[x2b_mask]

    # Limit to max of 1.0
    deadfrac[deadfrac > 1.0] = 1.0

    # Fire potential estimate
    fpi = (1.0 - deadfrac) * dr * 100.0
    # knots to mph
    ws *= 1.15078
    fpi = (fpi + (fpi * ws / 35.0) * (windreduction / 10.0) + 0.5)
    fpi[fpi > 240] = 240
    fpi = fpi.astype(numpy.uint8)

    fpi_file[valid_pixel_mask] = fpi[valid_pixel_mask]
    fpi_file[extm > 0.0] = fpi[extm > 0.0]

    # Assign values to pixel WFPI was not calculted for
    fpi_file[fuel_model == 13] = 251  # Ag Land
    fpi_file[fuel_model == 23] = 252  # Barren
    fpi_file[fuel_model == 24] = 253  # Marsh
    fpi_file[fuel_model == 0] = 249  # Outside US
    # Water
    fpi_file[numpy.logical_or(fuel_model == 22, greenness == 254)] = 254

    fpi_file.tofile(out_filename)

    if verbose:
        print("{} created".format(out_filename))


if __name__ == '__main__':

    parser = argparse.ArgumentParser(description="WFPI program")
    parser.add_argument('param_file', help='Parameter file')
    parser.add_argument(
        '-V', '--verbose', action='store_true', help='Print status messages')

    args = parser.parse_args()

    fpi10h_mso(args.param_file, args.verbose)
